#include "TROOT.h"
#include "TTree.h"
#include "TChain.h"
#include "TFile.h"
#include "TH2F.h"
#include <string>  
#include <fstream> 
#include <sstream>
#include <iostream>
#include <vector>

using namespace std;

///////////////////////////////////////////////////////////////////////////////////////
//
// C-Program to display events taken by the 252 detector stations of the KASCADE-Array
//
// Displayed are:
//   - Energy deposits in the e/gamma-detectors per station
//   - Energy deposits in the mu-detectors per station
//   - distribution of the arrival times per station
//   - event information box
//
// Input:
//   - ROOT file holding events measured by KASCADE downloaded from KCDC
//
// Called from root with
//    root -l
//    .L EventDisplay_KASCADE-raw.C
//    run()
//
// file: 'EventDisplay_KASCADE-raw.C'                                      
//                                                               J.Wochele 28.11.2022
//////////////////////////////////////////////////////////////////////////////////////

int run(void){
    gROOT->Reset();
    gStyle->SetOptStat(0);
    gStyle->SetPalette(55);
    gROOT->ProcessLine( "gErrorIgnoreLevel = 1001;");   // suppress ROOT warnings/Info

//////////////////////////////////////////////////////////////////////////////////////
// inputs required:
// provide root file from KCDC DataShop or Preselections
// to display random events set 'single_switch' and 'cut_switch' to 'false'
// to display one event set 'single_switch' to 'true' and give run- and event-numbers
// to display events selected by cuts set 'cut_switch' to 'true' and define your cuts
// within the code below (search for 'cut_switch')
// to read only a given number of events set 'ievents' to the desired number
    
    std::string path_to_data = "./example_KASCADE-raw.root";

    bool single_switch = false;  // display only one single Event (give run and event numbers)
    int irun=5008;               // input run number to be displayed
    int iev=50830;               // input event number to be displayed

    bool cut_switch    = false;  // display events matching cuts (see below)
    int ievents=0;               // number of events to be read from file (0:all)
    
//////////////////////////////////////////////////////////////////////////////////////

    std::cout<<"Input file:"<<path_to_data<<"\n";
     
    TChain array("array");
    TChain general("general");
    TChain row("row_map");
    array.Add(path_to_data.c_str());
    general.Add(path_to_data.c_str());
    row.Add(path_to_data.c_str());

    int nentries = row.GetEntries();

  // general declarations
    int        general_id;
    int        run = 0;          // run number
    int        ev  = 0;          // event number
    int        gt;               // global event time
    double     p;                // Air Pressure
    double     t;                // Air Temperature
 
  // declarations of leaf types: array (KASCADE)
    int    array_id;
    double xc    = 0.;           // x-core reconstructed by KASCADE
    double yc    = 0.;           // y-cor reconstructed by KASCADE
    double az    = 0.;           // Az reconstructed by KASCADE
    double ze    = 0.;           // Ze reconstructed by KASCADE
    double ne    = 0.;           // number e/g [Size in ntuple] [log10]
    double age   = 0.;           // age reconstructed by KASCADE
    double nmu   = 0.;           // nr of muons reconstructed by KASCADE [log10]
    double en    = 0.;           // estimated primary energy [log10]
 
    int    edepositn;            // number of active e/g stations 
    int    edeposits[252];       // Station ID active e/g-Stations [edepositn]
    double edeposit[252];        // energy deposit of e/g stations [edepositn]
    int    mdepositn;            // number of active mu stations 
    int    mdeposits[192];       // Station ID active mu-Stations [mdepositn]
    double mdeposit[192];        // energy deposit of mu-stations [mdepositn]
    int    arrivaln;             // number of active AT stations 
    int    arrivals[252];        // Station ID particip AT-Stations [arrivaln]
    int    arrival[252]={-1000}; // arrival times to be filled in [arrivaln] 
    int    arrival1[252];        // aux variable
    char   datetime[20];         // Event Date and Time


    array.SetBranchStatus("*",  0);
    general.SetBranchStatus("*",0);
 
    row.SetBranchStatus("array",  1);
    row.SetBranchStatus("general",1);

    row.SetBranchAddress("array",   &array_id);
    row.SetBranchAddress("general", &general_id);

    general.SetBranchStatus("R",  1);
    general.SetBranchStatus("Ev", 1);
    general.SetBranchStatus("P",  1);
    general.SetBranchStatus("T",  1);
    general.SetBranchStatus("Gt", 1);

    array.SetBranchStatus("Xc",   1);
    array.SetBranchStatus("Yc",   1);
    array.SetBranchStatus("Az",   1);
    array.SetBranchStatus("Ze",   1);
    array.SetBranchStatus("Ne",   1);
    array.SetBranchStatus("Age",  1);
    array.SetBranchStatus("Nmu",  1);
    array.SetBranchStatus("E",    1);

    general.SetBranchStatus("EDepositN",  1);
    general.SetBranchStatus("EDeposit",   1);
    general.SetBranchStatus("EDepositS",  1);
    general.SetBranchStatus("MDepositN",  1);
    general.SetBranchStatus("MDeposit",   1);
    general.SetBranchStatus("MDepositS",  1);
    general.SetBranchStatus("ArrivalN",   1);
    general.SetBranchStatus("Arrival",    1);
    general.SetBranchStatus("ArrivalS",   1);
    general.SetBranchStatus("Datetime",   1);
    
    general.SetBranchAddress("R",  &run);
    general.SetBranchAddress("Ev", &ev);
    general.SetBranchAddress("Gt", &gt); 
    general.SetBranchAddress("P",  &p); 
    general.SetBranchAddress("T",  &t); 

    array.SetBranchAddress("Xc",   &xc);
    array.SetBranchAddress("Yc",   &yc);
    array.SetBranchAddress("Az",   &az);
    array.SetBranchAddress("Ze",   &ze);
    array.SetBranchAddress("Ne",   &ne);
    array.SetBranchAddress("Age",  &age);
    array.SetBranchAddress("Nmu",  &nmu);
    array.SetBranchAddress("E",    &en);

    general.SetBranchAddress("EDepositN",  &edepositn);
    general.SetBranchAddress("EDeposit",   &edeposit);
    general.SetBranchAddress("EDepositS",  &edeposits);
    general.SetBranchAddress("MDepositN",  &mdepositn);
    general.SetBranchAddress("MDeposit",   &mdeposit);
    general.SetBranchAddress("MDepositS",  &mdeposits);
    general.SetBranchAddress("ArrivalN",   &arrivaln);
    general.SetBranchAddress("Arrival",    &arrival);
    general.SetBranchAddress("ArrivalS",   &arrivals);
    general.SetBranchAddress("Datetime",   &datetime);

    int Neg;
    int Nm;
    string Date, Time;
    char *s = new char[1];
    bool single_found = false;
    
  // Create histograms
    TH2F *hStNr = new TH2F("hStNr","", 16,0,16, 16,0,16);  // Station ID histogram
    TH2F *hEnDepoEG = new TH2F("hEnDepoEG", "", 16,0,16,16,0,16);   // e/g energy deposit
    TH2F *hEnDepoMU = new TH2F("hEnDepoMU", "", 16,0,16, 16,0,16);  // mu-energy deposit 
    TH2F *hArrivalT = new TH2F("hArrivalT", "", 16,0,16, 16,0,16);  // Arrival Times
    
  // define station ID of the KASCADE stations
    int StID[256] =
    {      
    1,   2,    5,   6,  17,  18,  21,  22,  33,  34,  37,  38,  49,  50,  53,  54, 
    4,   3,    8,   7,  20,  19,  24,  23,  36,  35,  40,  39,  52,  51,  56,  55, 
    13,  14,   9,  10,  29,  30,  25,  26,  45,  46,  41,  42,  61,  62,  57,  58,
    16,  15,  12,  11,  32,  31,  28,  27,  48,  47,  44,  43,  64,  63,  60,  59,
 
    65,  66,  69,  70,  81,  82,  85,  86,   96,  97, 100, 101, 111, 112, 115, 116,
    68,  67,  72,  71,  84,  83,  88,  87,   99,  98, 103, 102, 114, 113, 118, 117, 
    77,  78,  73,  74,  92,  93,  89,  90,  108, 109, 104, 105, 123, 124, 119, 120,
    80,  79,  76,  75,  95,  94,  91,   0,    0, 110, 107, 106, 126, 125, 122, 121, 
 
    127, 128, 131, 132, 143, 144, 147,   0,   0, 158, 161, 162, 173, 174, 177, 178,
    130, 129, 134, 133, 146, 145, 149, 148, 160, 159, 164, 163, 176, 175, 180, 179,
    139, 140, 135, 136, 154, 155, 150, 151, 169, 170, 165, 166, 185, 186, 181, 182,
    142, 141, 138, 137, 157, 156, 153, 152, 172, 171, 168, 167, 188, 187, 184, 183,

    189, 190, 193, 194, 205, 206, 209, 210, 221, 222, 225, 226, 237, 238, 241, 242, 
    192, 191, 196, 195, 208, 207, 212, 211, 224, 223, 228, 227, 240, 239, 244, 243,
    201, 202, 197, 198, 217, 218, 213, 214, 233, 234, 229, 230, 249, 250, 245, 246, 
    204, 203, 200, 199, 220, 219, 216, 215, 236, 235, 232, 231, 252, 251, 248, 247 
    };

  // define display axes  
    hEnDepoEG->GetXaxis()->SetNdivisions(810);
    hEnDepoEG->GetXaxis()->SetTitle("Eastern direction #rightarrow   ");
    hEnDepoEG->GetXaxis()->SetTitleOffset(1.6);
    hEnDepoEG->GetYaxis()->SetNdivisions(810);
    hEnDepoEG->GetYaxis()->SetTitle("#leftarrow Northern direction   ");
    hEnDepoEG->GetYaxis()->SetTitleOffset(1.8);

    hEnDepoMU->GetXaxis()->SetNdivisions(810);
    hEnDepoMU->GetXaxis()->SetTitle("Eastern direction #rightarrow   ");
    hEnDepoMU->GetXaxis()->SetTitleOffset(1.6);
    hEnDepoMU->GetYaxis()->SetNdivisions(810);
    hEnDepoMU->GetYaxis()->SetTitle("#leftarrow Northern direction   ");
    hEnDepoMU->GetYaxis()->SetTitleOffset(1.8);

    hArrivalT->GetXaxis()->SetNdivisions(810);
    hArrivalT->GetXaxis()->SetTitle("Eastern direction #rightarrow   ");
    hArrivalT->GetXaxis()->SetTitleOffset(1.6);
    hArrivalT->GetYaxis()->SetNdivisions(810);
    hArrivalT->GetYaxis()->SetTitle("#leftarrow Northern direction   ");
    hArrivalT->GetYaxis()->SetTitleOffset(1.8);

    
  // Fill array with station IDs (and draw)
    for(int j=0; j<16; j++){
      for(int k=0; k<16; k++){
        int n = 16*j + k;
        hStNr->SetBinContent(k+1, j+1, 0);
        hStNr->SetBinContent(k+1, j+1, StID[n]);
      }
    }
    ostringstream title;
    title.str("");
    ostringstream ofile;
    ofile.str("");
    ostringstream infofile;
    infofile.str("");
    ostringstream label;
    label.str("");
    ofstream inffile;
    
       
  // get entries and define Canvas
    std::cout << "Total number of events in file:  " <<nentries<<endl;
    
    auto c2 = new TCanvas("c2","Single Event Display",200,100,850,800);
    c2->Divide(2,2);

    if (ievents!=0) {
      nentries=ievents;
    }
    std::cout << "KASCADE events to be displayed: " << nentries <<endl;

    
  ////////////////////////////////////////////////////////////////////////////////  
  // entries loop

    for(int i=0; i<nentries; ++i){
      c2->Clear();
      c2->Divide(2,2);
      row.GetEntry(i);
      
  // get  branch_id's
      if (general_id<0) {
	continue;
      }
      if (general_id>=0) {
	general.GetEntry(general_id);
      }
      if (array_id>=0) {
	array.GetEntry(array_id);
      }
      
  // ========================================================================
  // display array data
      if (array_id>=0) {

	Date.assign(datetime,datetime+10);
	Time.assign(datetime+11,datetime+19);

  // mode to display a single event
      if (single_switch ) {
	if (run != irun || ev != iev ) {
	  continue;
	} else {
	  single_found = true;
	}	  
      }

  // mode to display events matching a cut
      if (cut_switch ) {
	if ( en <= 16.6 ) {
	  continue;
	}
      }
      std::cout << "displaying ARRAY event Run No: "<<run<<"  Event: "<<ev<<endl;
   
  //----------------------------------------------------------
  // display e/g energy deposits (Box 1)
	title.str("");
	title << "e/#gamma energy deposits in [eV]";
	hEnDepoEG->SetTitle(title.str().c_str());
	c2->cd(1);
	c2->SetLeftMargin(0.15);     // distance to plot left
	c2->SetBottomMargin(0.15);   // distance to plot bottom
	c2->SetRightMargin(0.15);    // distance to plot right
	c2->SetTopMargin(0.15);      // distance to plot top
	c2->SetGridx();
	c2->SetGridy();
	
	Neg = 0;
	for(int j=0; j<16; j++){
	  for(int k=0; k<16; k++){
	    int n = 16*j + k;
	    int StInd = StID[n];                       // 1-252
	    hEnDepoEG->SetBinContent(k+1, j+1, 0.);    // clear array
	    
	    for(int jeg=0; jeg<edepositn; jeg++){
	      if ((edeposit[jeg] >= 1.0) && (edeposit[jeg] <= 20000.0)) {
		if (edeposits[jeg]==StInd) {
		  hEnDepoEG->SetBinContent(k+1, j+1, edeposit[jeg]);
		  Neg = Neg + 1;
		}
	      }
	    }
	  }
	}
      //gPad->SetLogz(1);
      //hEnDepoEG->Draw("colz");     // draw 2d-color plot
	hEnDepoEG->Draw("lego2z");     // draw lego plot


  //-------------------------------------------------------
  // display mu energy deposits (Box 2)
      title.str("");
      title << "#mu energy deposits in [eV]";
      hEnDepoMU->SetTitle(title.str().c_str());
      c2->cd(2);    
      
      Nm = 0;
      for(int j=0; j<16; j++){
        for(int k=0; k<16; k++){
          int n = 16*j + k;
          int StInd = StID[n];                  // 1-252
          hEnDepoMU->SetBinContent(k+1, j+1, 0.); // clear data array

          for(int jmu=0; jmu<mdepositn; jmu++){
	    if ((mdeposit[jmu] >= 1.0) && (mdeposit[jmu] <= 600.0)) {
	      if (mdeposits[jmu]==StInd) {
	        hEnDepoMU->SetBinContent(k+1, j+1, mdeposit[jmu]);
		Nm = Nm + 1;
	      }
	    }
          }
        }
      }
      //gPad->SetLogz(1);
      //hEnDepoMU->Draw("colz");
      hEnDepoMU->Draw("lego2z");


  //----------------------------------------------------------
  // display arrival times (Box 3)
      title.str("");
      title << "Arrival time distribution in [ns]";
      hArrivalT->SetTitle(title.str().c_str());
      c2->cd(3);

      for(int j=0; j<16; j++){
	for(int k=0; k<16; k++){
	  int n = 16*j + k;
	  int StInd = StID[n];
	  hArrivalT->SetBinContent(k+1, j+1, 0.);   // clear data array

	  for(int jat=0; jat<arrivaln; jat++){
	    arrival1[StInd] = 0.;
	    if (arrival[jat] >= -200) {
	      if (arrivals[jat]==StInd) {
		arrival1[StInd] = arrival[jat];
		hArrivalT->SetBinContent(k+1, j+1, arrival1[StInd]);
	      }
	    }
	  }
	}
      }
  
      //hArrivalT->Draw("colz");
      hArrivalT->Draw("lego2z");

      
  // ========================================================================
  // Show event information (Box 4)
      c2->cd(4);
      TLatex l;
      l.SetTextSize(0.07);
      l.SetTextFont(72);
      l.SetTextColor(kBlack);
      l.SetNDC(false);
      const double xLeft  = 0.15;
      const double xRight = 0.85;
      const double yTop = 0.85;
      const double dY = 0.06;
      double y = 0.90;      
      double x = 0.12;

  // draw info headline
      l.DrawLatex(x, y, ("KASCADE Event Information"));
      l.SetTextSize(0.045);
      l.SetTextFont(42);

  // draw quantity names - adjust left
      y = yTop;
      l.SetTextAlign(12);
      vector<string> columnLabels = { "Run Nr.", "Event Nr.", "lg(E_{o})", "Zenith Angle", "Azimuth Angle", "X-Core Position", "Y-Core Position", "lg(N_{e})", "lg(N_{#mu})", "Number e/#gamma-Stations", "Number #mu-Stations", "Number timing Stations"};
      for (const auto& col : columnLabels) {
	l.DrawLatex(xLeft, y, col.c_str());
	y -= dY;
      }

  // draw quantities - adjust right
      y = yTop;
      l.SetTextAlign(32);
      vector<double> values;
      double xRun = (double)run;
      values.push_back(xRun);
      double xEv = (double)ev;
      values.push_back(xEv);
      values.push_back(en);
      values.push_back(ze);
      values.push_back(az);
      values.push_back(xc);
      values.push_back(yc);
      values.push_back(ne);
      values.push_back(nmu);
      double xNe = (double)Neg;
      values.push_back(xNe);
      double xNmu = (double)Nm;
      values.push_back(xNmu);
      double xTime = (double)arrivaln;
      values.push_back(xTime);

      vector<string> units = {"", "", " eV", "^{o}", "^{o}", " m", " m", "", "", "", "", ""};
      
      for (unsigned int iRow = 0; iRow < values.size(); ++iRow) {
	ostringstream cell;
	cell << fixed << setprecision((iRow <2 || iRow >8) ? 0 : 2) << values[iRow] << units[iRow];
	l.DrawLatex(xRight, y, cell.str().c_str());
	y -= dY;
      }
       
  // draw date and time strings
      double yTop1 = y;
      l.SetTextAlign(12);
      vector<string> columnLabels1 = { "Event Date", "Event Time"};
      for (const auto& col : columnLabels1) {
	l.DrawLatex(xLeft, y, col.c_str());
	y -= dY;
      }
      y = yTop1;
      l.SetTextAlign(32);
      vector<string> datim;
      datim.push_back(Date);
      datim.push_back(Time);
      for (unsigned int iRow = 0; iRow < datim.size(); ++iRow) {
	ostringstream cell;
	cell  << datim[iRow];
	l.DrawLatex(xRight, y, cell.str().c_str());
	y -= dY;
      }

      	c2->Update();

  // ========================================================================
  // print picture as .svg and event info file .info  

      ofile.str("");
      ofile << "ED_KA_R" << std::setfill('0') << std::setw(4)<< run << "E"  << std::setfill('0') << std::setw(7) << ev;
      c2->Print(("./" + ofile.str() + ".svg").c_str());
      
  // print event info as txt file
      infofile.str("");
      infofile << "ED_KA_R" << std::setfill('0') << std::setw(4)<< run << "E"  << std::setfill('0') << std::setw(7) << ev;
      inffile.open("./" + infofile.str() + ".info");

      inffile << "Event Display KASCADE - Event information\n";
      inffile << "-------------------------------------------\n";
      inffile << "Run Number             " << run << " \n";
      inffile << "Event Number           " << ev << " \n";
      inffile << "Prim Energy (lg10)     " << fixed << setprecision(2) << en<< " eV\n";
      inffile << "Zenith Angle           " << ze << "\u00B0\n";
      inffile << "Azimuth Angle          " << az << "\u00B0\n";
      inffile << "X-Core Position        " << xc << " m\n";
      inffile << "Y-Core Position        " << yc << " m\n";
      inffile << "lg(Ne)                 " << ne << "\n";
      inffile << "lg(Nmu)                " << nmu << "\n";
      inffile << "Number e/\u03B3-Stations    " << Neg << "\n";
      inffile << "Number \u03BC-Stations      " << Nm << "\n";
      inffile << "Number timing stations " << arrivaln << "\n";
      inffile << "Shower Age             " << age << "\n";
      inffile << "Event Time             " << Date << " " << Time << "\n";
      inffile << "Global Event Time      " << gt  << " (secs since 1.1.1970)\n";
      inffile << "Air Pressure           " << p  << " hPa\n";
      inffile << "Air Temperature        " << t  << "\u00B0\n\n";
      inffile.close();
      	
      c2->Update();

  // get next event or quit
      if ( single_found ) {          // single event found
	return 0;
      }
      char* gets(s);                       // hit <return> to continue
      if (string(s)=='q')            // press 'q' to quit
	return 0;  
     
      }
    }
    return 0;
}
